import { ContractType, AuthInput, Implementations, KeyValueStoreTypes, HandleResult } from './globalTypes.js';
import { AbstractBackend } from './backendAbstract.js';
export declare type ValueType = string | ArrayBuffer | ArrayBufferView | ReadableStream;
export declare type KvDataTypes = 'text' | 'json' | 'arrayBuffer' | 'stream';
export declare type GetResultType<T> = T extends undefined ? string : T extends 'text' ? string : T extends 'json' ? object : T extends 'arrayBuffer' ? ArrayBuffer : T extends 'stream' ? ReadableStream : never;
export declare type ListEntry = {
    name: string;
    expiration?: number;
    metadata: object;
};
export declare type KvListReturn = {
    keys: ListEntry[];
    list_complete: boolean;
    cursor: string;
};
export declare type KV = {
    list: (options?: {
        prefix?: string;
        limit?: number;
        cursor?: string;
    }) => Promise<KvListReturn>;
    get: (<T extends KvDataTypes>(key: string, type?: T) => Promise<GetResultType<T> | null>);
    getWithMetadata: (<T extends KvDataTypes>(key: string, type?: T) => Promise<{
        value: GetResultType<T> | null;
        metadata: object | null;
    }>);
    put: (key: string, value: ValueType, additional?: {
        metadata?: any;
        expiration?: number;
        expirationTtl?: number;
    }) => Promise<void>;
    delete: (key: string) => Promise<void>;
};
/**
 * Global variable to set custom key value store implementation
 * **/
export declare var customKv: {
    [key: string]: () => KV;
};
export declare const client: (key: KeyValueStoreTypes) => KV;
export declare const destroyAllClients: () => void;
export declare const destroyClient: (key: string) => void;
export declare const init: (key: any) => KV;
declare type KVi = Implementations.keyValue;
export declare const get: <IN, OUT>(contract: ContractType<"GET", Implementations.keyValue, IN, OUT>, auth: AuthInput, idIn: undefined | string | string[], input?: IN | undefined) => Promise<HandleResult<OUT>>;
export declare const post: <IN, OUT>(contract: ContractType<"POST", Implementations.keyValue, IN, OUT>, auth: AuthInput, id: string | undefined, body: IN) => Promise<HandleResult<OUT>>;
export declare const del: <IN, OUT>(contract: ContractType<"DELETE", Implementations.keyValue, IN, OUT>, auth: AuthInput, id: string | string[]) => Promise<HandleResult<OUT>>;
export declare const patch: <IN, OUT>(contract: ContractType<"PATCH", Implementations.keyValue, IN, OUT>, auth: AuthInput, id: string, body: IN) => Promise<HandleResult<OUT>>;
export declare const put: <IN, OUT>(contract: ContractType<"PUT", Implementations.keyValue, IN, OUT>, auth: AuthInput, id: string, body: IN) => Promise<HandleResult<OUT>>;
export declare const getKvProvider: () => AbstractBackend<Implementations.keyValue>;
export {};
