"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getElasticsearchProvider = exports.put = exports.patch = exports.del = exports.post = exports.get = exports.defaultSize = exports.createIndex = exports.info = exports.init = exports.destroyClient = exports.client = void 0;
const elasticsearch_1 = __importDefault(require("@elastic/elasticsearch"));
const uuid_1 = require("uuid");
const microtil_1 = require("microtil");
const backendAbstract_js_1 = require("./backendAbstract.js");
const Client = elasticsearch_1.default.Client;
let clientInstance;
exports.client = () => clientInstance || exports.init();
exports.destroyClient = () => {
    clientInstance = undefined;
};
exports.init = () => {
    const node = process.env.ELASTIC_HOST;
    const username = process.env.ELASTIC_USER_NAME;
    const password = process.env.ELASTIC_PASSWORD;
    const apiKey = process.env.ELASTIC_API_KEY;
    const apiId = process.env.ELASTIC_API_ID;
    const unauthenticated = process.env.ELASTIC_UNAUTHENTICATED;
    const setup = { node, requestTimeout: 90000 };
    if (username && password) {
        setup.auth = { username, password };
    }
    else if (apiKey) {
        setup.auth = apiId ? { apiKey: { id: apiId, api_key: apiKey } } : { apiKey };
    }
    else if (!unauthenticated) {
        console.warn('Elasticsearch api credentials are not set');
    }
    clientInstance = new Client(setup);
    return clientInstance;
};
exports.info = () => exports.client().info();
exports.createIndex = (name) => exports.client().indices.create({ index: name });
exports.defaultSize = 64;
const getById = (index, id) => __awaiter(void 0, void 0, void 0, function* () {
    const { body: { _source } } = yield exports.client().get({ index, id });
    return _source;
});
const getByIdChecked = (index, id, authentication, auth, manageFields) => __awaiter(void 0, void 0, void 0, function* () { return backendAbstract_js_1.filterToAccess([yield getById(index, id)], authentication, auth, manageFields); });
exports.get = (contract, auth, idIn, input) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b;
    const index = contract.implementation.index.toLowerCase();
    const { manageFields, authentication: authDef } = contract;
    const userIdFilter = {
        match: {
            createdBy: auth.sub || ''
        }
    };
    const id = idIn || ((_a = input) === null || _a === void 0 ? void 0 : _a.id);
    const search = (_b = input) === null || _b === void 0 ? void 0 : _b.search;
    if (Array.isArray(id)) {
        if (id.length === 0)
            return { result: [] };
        const { body: { docs } } = yield exports.client().mget({ index, body: { ids: id } });
        return { result: backendAbstract_js_1.filterToAccess(microtil_1.mapFilter(docs, (x) => x._source), authDef, auth, manageFields) };
    }
    else if (id) {
        try {
            const got = yield getByIdChecked(index, id, authDef, auth, manageFields);
            if (got.length === 0) {
                return backendAbstract_js_1.forbidden(id);
            }
            return { result: got };
        }
        catch (got) {
            if (got.meta.statusCode === 404)
                return backendAbstract_js_1.notFound({ id, input });
            throw got;
        }
    }
    else if (search) {
        const queryString = {
            query: {
                bool: {
                    must: [{ simple_query_string: { query: search } }]
                }
            }
        };
        if (!backendAbstract_js_1.authorizedByPermission(authDef, auth))
            queryString.query.bool.must.push(userIdFilter);
        const all = yield exports.client().search({
            index,
            body: queryString,
            size: contract.implementation.maxResults || exports.defaultSize
        });
        return { result: new Array(all.body.hits.hits).flatMap((y) => y.map((x) => x._source)) };
    }
    const searchAll = { index, size: contract.implementation.maxResults || exports.defaultSize };
    if (!backendAbstract_js_1.authorizedByPermission(authDef, auth)) {
        searchAll.body = {
            query: {
                bool: {
                    must: [userIdFilter]
                }
            }
        };
    }
    const all = yield exports.client().search(searchAll);
    const result = new Array(all.body.hits.hits).flatMap((y) => y.map((x) => x._source));
    // console.log('IN THE END', result)
    return { result: result };
});
exports.post = (contract, auth, id, body) => __awaiter(void 0, void 0, void 0, function* () {
    const idNew = id || uuid_1.v4();
    const newBody = Object.assign({}, body);
    if (contract.manageFields.id === true) {
        newBody.id = idNew;
    }
    if (contract.manageFields.createdBy === true) {
        newBody.createdBy = auth.sub;
    }
    try {
        yield exports.client().create({
            id: idNew,
            index: contract.implementation.index.toLowerCase(),
            refresh: 'wait_for',
            body: newBody
        });
    }
    catch (e) {
        if (e.meta.statusCode === 409)
            return { errorType: 'conflict', data: body, status: 409, errors: [] };
        throw e;
    }
    return { result: newBody };
});
exports.del = (contract, auth, id) => __awaiter(void 0, void 0, void 0, function* () {
    const index = contract.implementation.index.toLowerCase();
    if (Array.isArray(id)) {
        const data = yield Promise.all(id.map((x) => __awaiter(void 0, void 0, void 0, function* () { return ({ id, result: yield exports.del(contract, auth, x) }); })));
        const errors = data.reduce((p, c) => p.concat(Array.isArray(c.result.errors) && c.result.errors.length
            ? c.result.errors
            : []), []);
        if (errors.length) {
            return { errorType: 'forbidden', data, status: 403, errors };
        }
        return { result: {} };
    }
    try {
        const result = yield getByIdChecked(index, id, contract.authentication, auth, contract.manageFields);
        if (!result || result.length === 0) {
            return backendAbstract_js_1.forbidden(id, ['Can\'t delete, unauthorized']);
        }
        yield exports.client().delete({ index, id, refresh: 'wait_for' });
    }
    catch (error) {
        if (error.meta.statusCode === 404)
            return { errorType: 'notFound', status: 404, data: id, errors: [] };
        throw error;
    }
    return { result: {} };
});
exports.patch = (contract, auth, id, body) => __awaiter(void 0, void 0, void 0, function* () {
    const index = contract.implementation.index.toLowerCase();
    try {
        const result = yield getByIdChecked(index, id, contract.authentication, auth, contract.manageFields);
        if (!result || result.length === 0)
            return backendAbstract_js_1.forbidden({ id, body });
        let newBody = body;
        if (contract.manageFields.createdBy === true) {
            newBody = Object.assign({}, body);
            newBody.createdBy = result[0].createdBy;
        }
        yield exports.client().update({
            index,
            refresh: 'wait_for',
            id,
            body: { doc: newBody }
        });
    }
    catch (error) {
        if (error.meta.statusCode === 404)
            return backendAbstract_js_1.notFound({ id, body });
        throw error;
    }
    return { result: {} };
});
exports.put = (contract, auth, id, body) => __awaiter(void 0, void 0, void 0, function* () {
    const index = contract.implementation.index.toLowerCase();
    try {
        const result = yield getByIdChecked(index, id, contract.authentication, auth, contract.manageFields);
        if (!result || result.length === 0)
            return backendAbstract_js_1.forbidden({ id, body });
        let newBody = body;
        if (contract.manageFields.createdBy === true) {
            newBody = Object.assign({}, body);
            newBody.createdBy = result[0].createdBy;
        }
        yield exports.client().index({
            index,
            refresh: 'wait_for',
            id,
            body: newBody
        });
        return { result: {} };
    }
    catch (error) {
        if (error.meta.statusCode === 404)
            return backendAbstract_js_1.notFound({ id, body });
        throw error;
    }
});
exports.getElasticsearchProvider = () => ({
    get: exports.get,
    post: exports.post,
    put: exports.put,
    patch: exports.patch,
    delete: exports.del
});
//# sourceMappingURL=backendElasticsearch.js.map