"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getKvProvider = exports.put = exports.patch = exports.del = exports.post = exports.get = exports.init = exports.destroyClient = exports.destroyAllClients = exports.client = void 0;
const uuid_1 = require("uuid");
const memoryKv_js_1 = require("./memoryKv.js");
const workerKv_js_1 = require("./workerKv.js");
const backendAbstract_js_1 = require("./backendAbstract.js");
const clientInstance = {};
const typeToString = (input) => typeof input === 'string' ? input : input.custom;
exports.client = (key) => clientInstance[typeToString(key)] || exports.init(typeToString(key));
exports.destroyAllClients = () => {
    for (const key of Object.keys(clientInstance))
        delete clientInstance[key];
};
exports.destroyClient = (key) => {
    delete clientInstance[key];
};
exports.init = (key) => {
    if (key === 'worker') {
        clientInstance.worker = workerKv_js_1.workerKv();
        return clientInstance.worker;
    }
    if (key === 'memory') {
        clientInstance.memory = memoryKv_js_1.memoryKV();
        return clientInstance.memory;
    }
    if (typeof exports.customKv !== 'undefined') {
        if (key in exports.customKv) {
            clientInstance[key] = exports.customKv[key]();
            return clientInstance[key];
        }
    }
    throw new Error(`Unknown key value backend: '${key}'`);
};
const authorizedByPermission = (auth, authInput) => typeof auth === 'boolean' ||
    auth.some(x => (authInput.permissions || []).some(y => x === y));
const getUserIdFields = (fields) => Object.entries(fields).filter(x => x[1]).map(x => x[0]);
const filterToAccess = (input, auth, authInput, fields) => authorizedByPermission(auth, authInput) ? input : input.filter((x) => getUserIdFields(fields).some(y => x[y] === authInput.sub));
const keyId = (index, id) => `${index}:records:${id}`;
const getByIdChecked = (id, auth, type, index, authDef, manageFields) => __awaiter(void 0, void 0, void 0, function* () {
    const result = yield exports.client(type).get(keyId(index, id), 'json');
    return filterToAccess([result], authDef, auth, manageFields);
});
exports.get = (contract, auth, idIn, input) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b, _c;
    const id = idIn || ((_a = input) === null || _a === void 0 ? void 0 : _a.id);
    let cursor = (_b = input) === null || _b === void 0 ? void 0 : _b.cursor;
    const limit = ((_c = input) === null || _c === void 0 ? void 0 : _c.limit) || 64;
    const type = contract.implementation.backend;
    const index = contract.implementation.prefix;
    if (Array.isArray(id)) {
        if (id.length === 0)
            return { result: [] };
        const docs = (yield Promise.all(id.map(x => exports.client(type).get(keyId(index, x), 'json'))))
            .filter(x => x != null);
        return { result: filterToAccess(docs, contract.authentication, auth, contract.manageFields) };
    }
    else if (id) {
        const result = yield exports.client(type).get(keyId(index, id), 'json');
        if (!result)
            return backendAbstract_js_1.notFound({ id, input });
        const filtered = filterToAccess([result], contract.authentication, auth, contract.manageFields);
        if (filtered.length === 0)
            return backendAbstract_js_1.forbidden(input);
        return { result: filtered };
    }
    if (!contract.implementation.allowGetAll)
        return { errorType: 'badInput', status: 400, errors: ['Get all is disabled, id must be provided'] };
    const accessAll = authorizedByPermission(contract.authentication, auth);
    const listId = [];
    const result = yield exports.client(type)
        .list({ limit: Math.max(10, limit), cursor, prefix: `${index}:records` });
    result.keys.forEach((x) => __awaiter(void 0, void 0, void 0, function* () {
        if (accessAll || x.metadata.createdBy === auth.sub) {
            listId.push(exports.client(type).get(x.name, 'json'));
        }
    }));
    if (listId.length >= limit)
        cursor = null;
    if (result.list_complete)
        cursor = null;
    return {
        result: (yield Promise.all(listId)).filter((x) => x != null),
        cursor: result.cursor,
        more: !result.list_complete
    };
});
exports.post = (contract, auth, id, body) => __awaiter(void 0, void 0, void 0, function* () {
    const type = contract.implementation.backend;
    const index = contract.implementation.prefix;
    const newId = id || uuid_1.v4();
    const newBody = Object.assign({}, body);
    if (contract.manageFields.id === true) {
        newBody.id = newId;
    }
    const metadata = {};
    if (contract.manageFields.createdBy === true) {
        newBody.createdBy = auth.sub;
        metadata.createdBy = auth.sub;
    }
    // Maybe skip check if it is generated?
    const got = yield exports.client(type).get(keyId(index, newId));
    if (got)
        return { errorType: 'conflict', data: body, status: 409, errors: [] };
    // TODO returned without the full id, that contains the index, or maybe always remove the index when returning?
    yield exports.client(type).put(keyId(index, newId), JSON.stringify(newBody), { metadata });
    return { result: newBody };
});
exports.del = (contract, auth, id) => __awaiter(void 0, void 0, void 0, function* () {
    if (Array.isArray(id)) {
        const data = yield Promise.all(id.map((x) => __awaiter(void 0, void 0, void 0, function* () { return ({ id, result: yield exports.del(contract, auth, x) }); })));
        const errors = data.reduce((p, c) => p.concat(Array.isArray(c.result.errors) && c.result.errors.length
            ? c.result.errors
            : []), []);
        if (errors.length) {
            return { errorType: 'forbidden', data, status: 403, errors };
        }
        return { result: {} };
    }
    const type = contract.implementation.backend;
    const index = contract.implementation.prefix;
    const result = yield getByIdChecked(id, auth, type, index, contract.authentication, contract.manageFields);
    if (result.length === 1 && result[0] === null) {
        return backendAbstract_js_1.notFound({ id });
    }
    if (!result || result.length === 0)
        return backendAbstract_js_1.forbidden(id, [`forbidden - could not delete item: ${id} `]);
    yield exports.client(type).delete(keyId(index, id));
    return { result: {} };
});
exports.patch = (contract, auth, id, body) => __awaiter(void 0, void 0, void 0, function* () {
    const type = contract.implementation.backend;
    const index = contract.implementation.prefix;
    const result = yield getByIdChecked(id, auth, type, index, contract.authentication, contract.manageFields);
    if (!result || result.length === 0)
        return backendAbstract_js_1.forbidden({ id, body });
    const newBody = Object.assign({}, result[0]);
    for (const [key, value] of Object.entries(body)) {
        newBody[key] = value;
    }
    if (contract.manageFields.createdBy === true) {
        newBody.createdBy = result[0].createdBy;
    }
    const key = keyId(index, id);
    const { value, metadata } = yield exports.client(type).getWithMetadata(key);
    if (value == null) {
        return backendAbstract_js_1.notFound({ id, body });
    }
    yield exports.client(type).put(key, JSON.stringify(newBody), { metadata });
    return { result: {} };
});
exports.put = (contract, auth, id, body) => __awaiter(void 0, void 0, void 0, function* () {
    const type = contract.implementation.backend;
    const index = contract.implementation.prefix;
    const result = yield getByIdChecked(id, auth, type, index, contract.authentication, contract.manageFields);
    if (!result || result.length === 0)
        return backendAbstract_js_1.forbidden({ id, body });
    const newBody = Object.assign({}, body);
    if (contract.manageFields.createdBy === true) {
        newBody.createdBy = result[0].createdBy;
    }
    const key = keyId(index, id);
    const { value, metadata } = yield exports.client(type).getWithMetadata(key);
    if (value == null) {
        return backendAbstract_js_1.notFound({ id, body });
    }
    yield exports.client(type).put(key, JSON.stringify(newBody), { metadata });
    return { result: {} };
});
exports.getKvProvider = () => ({
    get: exports.get,
    post: exports.post,
    put: exports.put,
    patch: exports.patch,
    delete: exports.del
});
//# sourceMappingURL=backendKv.js.map