"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.wrapHandleWithValidation = void 0;
const yaschva_1 = require("yaschva");
const globalTypes_js_1 = require("./globalTypes.js");
const inputValidationFailed = (errors, data) => ({
    errorType: 'Input validation failed',
    data,
    status: 400,
    errors
});
const notImplemented = (contractName) => ({
    errorType: 'Not implemented',
    data: contractName,
    status: 501,
    errors: [`Handler for ${contractName} was not defined`]
});
const unexpectedResult = (errors, data) => ({
    errorType: 'Unexpected result from function',
    data,
    status: 500,
    errors
});
exports.wrapHandleWithValidation = (contract, validateOutput = true) => {
    return (input, auth = {}, id) => __awaiter(void 0, void 0, void 0, function* () {
        const validationResult = yaschva_1.validate(contract.arguments, input);
        if (validationResult.result === 'fail') {
            return inputValidationFailed(validationResult, input);
        }
        if (contract.handle) {
            const result = yield contract.handle(input, Object.assign({}, auth), contract, id);
            if (validateOutput && !globalTypes_js_1.isContractInError(result)) {
                const outputValidation = yaschva_1.validate(contract.returns, result.result);
                if (outputValidation.result === 'fail')
                    return unexpectedResult(outputValidation, result.result);
            }
            return result;
        }
        return notImplemented(contract.name);
    });
};
//# sourceMappingURL=contractValidation.js.map