"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.memoryKV = void 0;
function ab2str(buf) {
    return String.fromCharCode.apply(null, new Uint16Array(buf));
}
function str2ab(str) {
    var buf = new ArrayBuffer(str.length * 2); // 2 bytes for each char
    var bufView = new Uint16Array(buf);
    for (var i = 0, strLen = str.length; i < strLen; i++) {
        bufView[i] = str.charCodeAt(i);
    }
    return buf;
}
exports.memoryKV = () => {
    const db = new Map();
    const dbMeta = new Map();
    const list = (options) => __awaiter(void 0, void 0, void 0, function* () {
        let currentCursor = '';
        let cursorFound = false;
        const { cursor, limit, prefix } = options || {};
        const result = [];
        for (const value of dbMeta.values()) {
            if (cursor && !cursorFound) {
                if (value.name === cursor) {
                    cursorFound = true;
                }
                else {
                    continue;
                }
            }
            if (limit && result.length === limit) {
                currentCursor = value.name;
                break;
            }
            if (!prefix || value.name.startsWith(prefix)) {
                result.push(value);
            }
        }
        return {
            keys: result,
            cursor: currentCursor,
            list_complete: true
        };
    });
    const get = (key, type) => __awaiter(void 0, void 0, void 0, function* () {
        const returned = db.get(key);
        if (returned == null)
            return null;
        if (type === 'json')
            return JSON.parse(returned);
        else if (type === 'arrayBuffer')
            return str2ab(returned);
        else if (type === 'stream')
            return returned == null ? null : JSON.parse(returned);
        return returned;
    });
    const put = (key, value, additional) => __awaiter(void 0, void 0, void 0, function* () {
        const { expiration, expirationTtl } = additional || {};
        const exp = expiration || (expirationTtl ? (Date.now() / 1000) + expirationTtl : undefined);
        if (value instanceof ArrayBuffer) {
            db.set(key, ab2str(value));
        }
        else {
            db.set(key, typeof value === 'string' ? value : JSON.stringify(value));
        }
        if (additional) {
            dbMeta.set(key, { name: key, metadata: additional.metadata, expiration: exp });
        }
    });
    const destroy = (key) => __awaiter(void 0, void 0, void 0, function* () {
        db.delete(key);
        dbMeta.delete(key);
    });
    const getWithMetadata = (key, type) => __awaiter(void 0, void 0, void 0, function* () {
        var _a, _b;
        return {
            value: yield get(key, type),
            metadata: ((_b = (_a = (yield list({ prefix: key }))) === null || _a === void 0 ? void 0 : _a.keys[0]) === null || _b === void 0 ? void 0 : _b.metadata) || null
        };
    });
    return { list, get, put, delete: destroy, getWithMetadata };
};
//# sourceMappingURL=memoryKv.js.map