import { wrapHandleWithValidation } from './contractValidation.js';
import { map } from 'microtil';
function isPrimitive(test) {
    return (test !== Object(test));
}
;
export const errorStructure = (status, errorType, errors, data) => ({ status, errorType, data, errors: [errors] });
export const processHandle = (contract) => async (body, id, user) => {
    const { authentication, manageFields } = contract;
    if (authentication) {
        if (!(user === null || user === void 0 ? void 0 : user.sub)) {
            return errorStructure(401, 'unauthorized', 'Only logged in users can do this', { id });
        }
        if (Array.isArray(authentication)) {
            const perm = user.permissions || [];
            const hasPerm = perm.some(y => authentication.some(z => z === y));
            const canUserAccess = manageFields.createdBy;
            if (!hasPerm && !canUserAccess) {
                return errorStructure(403, 'forbidden', "You don't have permission to do this", { id });
            }
        }
    }
    try {
        const result = await wrapHandleWithValidation(contract)(body, { ...user }, id);
        return { ...result, status: result.status || (contract.type === 'POST' ? 201 : 200) };
    }
    catch (e) {
        const normalizeErrorCode = (input) => input >= 400 && input < 600 ? input : 500;
        const getIfNumber = (input) => typeof input === 'number' && normalizeErrorCode(input);
        const data = (isPrimitive(e) || !e) ? e : map(e, y => y);
        const code = getIfNumber(e === null || e === void 0 ? void 0 : e.status) || getIfNumber(e === null || e === void 0 ? void 0 : e.statusCode) || getIfNumber(e === null || e === void 0 ? void 0 : e.code) || 500;
        return errorStructure(code, (e === null || e === void 0 ? void 0 : e.name) || 'exception', (e === null || e === void 0 ? void 0 : e.message) || (e === null || e === void 0 ? void 0 : e.toString()) || 'unknown', data);
    }
};
//# sourceMappingURL=process.js.map