import { ExecutionContext } from 'ava';
import { ContractType, ManageableFields, AuthenticationDefinition, AuthInput, HttpMethods, Implementation, HandleResult, HandleResultSuccess, HandleErrorResponse, Implementations } from './globalTypes.js';
import { Validation } from 'yaschva';
import { AbstractBackend } from './backendAbstract.js';
export declare type TestContractOut = {
    id: string;
    b: string;
};
export declare type TestContractIn = {
    id?: string;
    a: string;
};
export declare type TestContractType<T extends HttpMethods, IMPL extends Implementation, IN, OUT> = ContractType<T, IMPL, IN, OUT>;
export declare type CONTRACT_COLLECTION<A extends Implementation> = {
    post: ContractType<'POST', A, any, any>;
    get: ContractType<'GET', A, any, any>;
    del: ContractType<'DELETE', A, any, any>;
    put: ContractType<'PUT', A, any, any>;
    patch: ContractType<'PATCH', A, any, any>;
};
export declare type ANY_CONTRACTS = CONTRACT_COLLECTION<Implementation>;
export declare type TestFn = (backend: AbstractBackend<any>, postContract: CONTRACT_COLLECTION<any>) => (t: ExecutionContext) => Promise<void>;
export declare const runTestArray: <A extends Implementation>(input: {
    contracts: CONTRACT_COLLECTION<A>;
    skip: string[];
}, tests: [string, TestFn][]) => void;
export declare const mockHandle: <METHOD extends HttpMethods, IMPL extends Implementation, IN, OUT>(input: IN, _: AuthInput, contract: ContractType<METHOD, IMPL, IN, OUT>) => Promise<HandleResult<OUT>>;
export declare const getContract: <T extends HttpMethods, IMPL extends Implementation = Implementations.manual, IN = TestContractIn, OUT = TestContractOut>(input?: {
    manageFields?: ManageableFields | undefined;
    authentication?: boolean | (string | {
        createdBy: boolean;
    })[] | undefined;
    name?: string | undefined;
    handle?: ((input: IN, auth: AuthInput, contract: ContractType<T, IMPL, IN, OUT>, id?: string | undefined) => Promise<HandleResult<OUT>>) | undefined;
    method?: T | undefined;
    implementation?: IMPL | undefined;
    arguments?: string | import("yaschva").EnumType | import("yaschva").ObjectType | import("yaschva").ArrayType | import("yaschva").ObjectMetaType | import("yaschva").StringType | import("yaschva").NumberType | import("yaschva").MetaType | import("yaschva").MapType | import("yaschva").ValueType[] | import("yaschva").TypeDef | undefined;
    returns?: string | import("yaschva").EnumType | import("yaschva").ObjectType | import("yaschva").ArrayType | import("yaschva").ObjectMetaType | import("yaschva").StringType | import("yaschva").NumberType | import("yaschva").MetaType | import("yaschva").MapType | import("yaschva").ValueType[] | import("yaschva").TypeDef | undefined;
}) => ContractType<T, IMPL, IN, OUT>;
export declare const ExpectGood: <OUT = unknown>(doAction: (backend: AbstractBackend<any>, contract: ANY_CONTRACTS, t: ExecutionContext) => Promise<HandleResult<OUT>>, asserts: (a: HandleResultSuccess<OUT>, t: ExecutionContext, contract: ANY_CONTRACTS) => void | Promise<void>) => (backend: AbstractBackend<any>, contract: ANY_CONTRACTS) => (t: ExecutionContext) => Promise<void>;
export declare const ExpectBad: <OUT = unknown>(doAction: (backend: AbstractBackend<any>, contract: ANY_CONTRACTS, t: ExecutionContext) => Promise<HandleResult<OUT>>, asserts: (a: HandleErrorResponse, t: ExecutionContext, contract: ANY_CONTRACTS) => void | Promise<void>) => (backend: AbstractBackend<any>, contract: ANY_CONTRACTS) => (t: ExecutionContext) => Promise<void>;
export declare const baseDataSchema: {
    a: string;
    b: string[];
};
export declare const contractCollection: () => {
    contracts: CONTRACT_COLLECTION<Implementation>;
    skip: string[];
}[];
export declare const postSome: <A extends unknown>(db: AbstractBackend<any>, contract: ContractType<"POST", any, any, A>, authInput?: AuthInput, num?: number) => Promise<HandleResult<A>[]>;
export declare const withAuth: <T extends ContractType<any, any, any, any>>(c: T) => T;
export declare const throwOnError: <A = unknown>(input: HandleResult<A>) => input is HandleResultSuccess<A>;
